package com.bweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.OutRechargePrice;
import com.hai.entity.SecDictionary;
import com.hai.model.ResponseData;
import com.hai.model.UserInfoModel;
import com.hai.service.CommonService;
import com.hai.service.OutRechargePriceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "/outRechargePrice")
@Api(value = "查询充值金额接口")
public class OutRechargePriceController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private OutRechargePriceService outRechargePriceService;

    @Resource
    private CommonService commonService;

    @RequestMapping(value = "/getListOutRechargePrice", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询充值金额列表")
    public ResponseData getListOutRechargePrice(
            @RequestParam(value = "operatorType", required = false) Integer operatorType,
            @RequestParam(value = "rechargePlatform", required = false) Integer rechargePlatform,
            @RequestParam(value = "rechargeType", required = false) Integer rechargeType,
            @RequestParam(value = "rechargeWay", required = false) Integer rechargeWay,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(name = "pageNum", required = true) Integer pageNum,
            @RequestParam(name = "pageSize", required = true) Integer pageSize, HttpServletRequest request
    ) {
        try {

            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();

            Map<String, Object> map = new HashMap<>();

            map.put("operatorType", operatorType);
            map.put("rechargePlatform", rechargePlatform);
            map.put("rechargeType", rechargeType);
            map.put("rechargeWay", rechargeWay);
            map.put("status", status);

            map.put("companyId", String.valueOf(userInfoModel.getBsCompany().getId()));

            PageHelper.startPage(pageNum, pageSize);

            List<OutRechargePrice> outRechargePrices = outRechargePriceService.getListRechargePrice(map);

            return ResponseMsgUtil.success(new PageInfo<>(outRechargePrices));
        } catch (Exception e) {
            log.error("OutRechargePriceController --> getListUser() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/insertPrice", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "新增金额")
    public ResponseData insertPrice(@RequestBody JSONObject object, HttpServletRequest request) {
        try {

            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();

            if (object == null ||
                    object.getJSONArray("rechargePrices") == null ||
                    object.getBigDecimal("discount") == null ||
                    object.getString("operatorType") == null ||
                    object.getJSONArray("rechargePlatform") == null ||
                    object.getJSONArray("productPlatform") == null ||
                    object.getInteger("rechargeType") == null ||
                    object.getJSONArray("payType") == null ||
                    object.getInteger("rechargeWay") == null ||
                    object.getBigDecimal("integralDiscount") == null
            ) {
                log.error("OutRechargePriceController -> insertPrice() error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            object.put("operatorId", userInfoModel.getSecUser().getId());
            object.put("operatorName", userInfoModel.getSecUser().getLoginName());
            object.put("companyId", userInfoModel.getBsCompany().getId());

            outRechargePriceService.insertRechargePrice(object);

            return ResponseMsgUtil.success("新增成功");

        } catch (Exception e) {
            log.error("OutRechargePriceController --> insertPrice() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/updatePrice", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改金额")
    public ResponseData updatePrice(@RequestBody JSONObject object, HttpServletRequest request) {
        try {

            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();

            if (object == null ||
                    object.getString("rechargePrices") == null ||
                    object.getLong("id") == null ||
                    object.getBigDecimal("discount") == null ||
                    object.getString("operatorType") == null ||
                    object.getString("rechargePlatform") == null ||
                    object.getInteger("rechargeType") == null ||
                    object.getString("payType") == null ||
                    object.getString("rechargeWay") == null ||
                    object.getString("platformType") == null ||
                    object.getBigDecimal("integralDiscount") == null
            ) {
                log.error("OutRechargePriceController -> insertPrice() error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            OutRechargePrice outRechargePrice = outRechargePriceService.findById(object.getLong("id"));

            if (outRechargePrice == null) {
                log.error("OutRechargePriceController -> updatePrice() error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.UPDATE_DATA_ERROR, "");
            }

            outRechargePrice.setPayPrice(outRechargePrice.getRechargePrice().multiply(object.getBigDecimal("discount").divide(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP)));
            outRechargePrice.setRechargeType(object.getInteger("rechargeType"));
            outRechargePrice.setOperatorType(object.getInteger("operatorType"));
            outRechargePrice.setRechargePlatform(object.getString("rechargePlatform"));
            outRechargePrice.setRechargeWay(object.getInteger("rechargeWay"));
            outRechargePrice.setUpdateTime(new Date());
            outRechargePrice.setStatus(3);

            outRechargePriceService.updateRechargePrice(outRechargePrice);

            JSONObject postObject = new JSONObject();
            postObject.put("payType", object.getString("payType"));
            postObject.put("integralDiscount", object.getBigDecimal("integralDiscount"));
            postObject.put("productPlatform", object.getString("productPlatform"));
            postObject.put("operatorId", userInfoModel.getSecUser().getId());
            postObject.put("operatorName", userInfoModel.getSecUser().getLoginName());
            postObject.put("sourceId", outRechargePrice.getId());

            // 配置支付方式
            commonService.configPayType(postObject);
            // 配置积分折扣比例
            commonService.configPayType(postObject);
            // 配置展示平台
            commonService.configPlatform(postObject);

            return ResponseMsgUtil.success("修改成功");

        } catch (Exception e) {
            log.error("OutRechargePriceController --> insertPrice() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询详情")
    public ResponseData findById(@RequestParam(value = "id", required = true) Long id) {
        try {

            OutRechargePrice outRechargePrice = outRechargePriceService.findById(id);
            return ResponseMsgUtil.success(outRechargePrice);

        } catch (Exception e) {
            log.error("OutRechargePriceController --> findById() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/deletePrice", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "删除")
    public ResponseData deletePrice(@RequestParam(value = "id", required = true) Long id) {
        try {

            outRechargePriceService.deletePrice(id);
            return ResponseMsgUtil.success("删除成功");

        } catch (Exception e) {
            log.error("OutRechargePriceController --> deletePrice() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/editPriceStatus", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "修改价格状态")
    public ResponseData editPriceStatus(@RequestParam(value = "id", required = true) Long id,
                                        @RequestParam(value = "status", required = true) Integer status) {
        try {

            OutRechargePrice outRechargePrice = outRechargePriceService.findById(id);
            outRechargePrice.setStatus(status);
            outRechargePriceService.updateRechargePrice(outRechargePrice);

            return ResponseMsgUtil.success("修改成功");

        } catch (Exception e) {
            log.error("OutRechargePriceController --> editPriceStatus() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
